<?php

/**
 * Notification Handling functionality of the plugin.
 *
 * @link       https://t.me/WPTelegram
 * @since      1.4.0
 *
 * @package    Wptelegram
 * @subpackage Wptelegram/includes
 */

/**
 * The Notification Handling functionality of the plugin.
 *
 *
 * @package    Wptelegram
 * @subpackage Wptelegram/includes
 * @author     Manzoor Wani <@manzoorwanijk>
 */
class Wptelegram_Notification_Handler {

	/**
	 * Settings/Options
	 *
	 * @since  	1.4.0
	 * @access 	private
	 * @var  	string 		$options 	Plugin Options
	 */
	private $options;

	/**
	 * phpmailer_action arguments
	 *
	 * @since  	1.4.0
	 * @access 	private
	 * @var  	array
	 */
	private $mailer_args;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.4.0
	 */
	public function __construct() {
		global $wptelegram_options;
		
		$this->options = $wptelegram_options;
	}

	/**
	 * Modify $phpmailer object
	 *
	 * @since	1.4.0
	 *
	 * @param	$phpmailer	PHPMailer
	 */
	public function handle_phpmailer_init( $phpmailer ) {
		$phpmailer->action_function = array( $this, 'handle_phpmailer_action' );
	}

	/**
	 * phpmailer action_function
	 *
	 * @since 1.4.0
	 *
	 * @param	boolean	$result
	 * @param	string	$to
	 * @param	string	$cc
	 * @param	string	$bcc
     * @param	string	$subject
     * @param	string	$body
     *
     * @access public
	 */
	public function handle_phpmailer_action( $result, $to, $cc, $bcc, $subject, $body ) {
		$this->mailer_args = func_get_args();
		
		$bot_token = $this->options['telegram']['bot_token'];
		$chat_ids = $this->options['notify']['chat_ids'];

		if ( ! ( $bot_token && $chat_ids ) ) {
			return;
		}

		$args = compact( 'result', 'to', 'cc', 'bcc', 'subject', 'body' );
		$send = apply_filters( 'wptelegram_send_notification', true, $args );

		if ( ! ( $send && $this->is_watched_recipient( $to ) ) ) {
			return;
		}
		$this->prepare_text( $subject, $body, $args );
	}

	/**
	 * prepare the text to be sent to Telegram
	 *
	 * @since 1.4.0
	 *
     * @param	string	$subject
	 * @param	string	$body
     * @param	array	$args Array of args passed to PHPMailer action function
     *
     * @access public
	 */
	public function prepare_text( $subject, $body, $args = '' ) {

		$text = convert_html_to_text( $body, true );

		$text = '🔔‌*' . $subject . '‌🔔*' . "\n\n" . $text;

		$hashtag = $this->options['notify']['hashtag'];
		if ( $hashtag ) {
			$text .= "\n\n" . $hashtag;
		}
		$text = apply_filters( 'wptelegram_notification_text', $text, $subject, $body );

		$method_params = array(
			'sendMessage' => array(
				'text'			=> $text,
				'parse_mode'	=> 'Markdown',
			),
		);
		$this->send( $method_params, $args );
	}

	/**
	 * Send the message
	 *
	 * @since 1.4.0
	 *
	 * @param 	array	$method_params	Array of methods
     * @param 	array 	$args			Array of args passed to PHPMailer action function
	 */
	public function send( $method_params, $args = '' ){

		$bot_token = $this->options['telegram']['bot_token'];
		$chat_ids = $this->options['notify']['chat_ids'];
		
		$tg_api = new WPTelegram_Telegram_Api( $bot_token );

		$chat_ids = explode( ',', $chat_ids );

		$chat_ids = (array) apply_filters( 'wptelegram_notification_chat_ids', $chat_ids, $args );

		foreach ( $chat_ids as $chat_id ) {
			
			foreach ( $method_params as $method => $params ) {
				$params['chat_id'] = $chat_id;
				
				$response = $tg_api->$method( $params );

				do_action( 'wptelegram_notification_response', $response, $this->mailer_args );
			}
		}
	}

	/**
	 * Check if the recipient of the email is in the watch list
	 *
	 * @since 1.4.0
	 *
	 * @param	string|array	$recipients
	 *
     * @return	boolean
     *
     * @access	private
	 */
	private function is_watched_recipient( $recipients ) {
		$watch_emails = $this->options['notify']['watch_emails'];

		if ( 'all' == $watch_emails ) {
			return true;
		}
		foreach ( (array) $recipients as $recipient ) {
			if ( is_array( $recipient ) && preg_match( '/' . $recipient[0] . '/iu', $watch_emails ) ) {
				return true;
			}
			else {
				$addresses = explode( ',', $recipient );
				foreach ( $addresses as $address ) {

					// Break $address into name and address parts if in the format "Name <user@example.com>"
	                if ( preg_match( '/(.*)<(.+?)>/', $address, $matches ) ) {
	                    if ( 3 == count( $matches ) ) {
	                        $address = $matches[2];
	                    }
	                }
					if ( preg_match( '/' . trim( $address ) . '/iu', $watch_emails ) ) {
						return true;
					}
				}
			}
		}
		return false;
	}
}